/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.datastructure
 * File:    IfmapDataType.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.datastructure;

import java.util.ArrayList;

public abstract class IfmapDataType {

	private ArrayList<Metadata> mMetadata;

	public IfmapDataType() {
		mMetadata = new ArrayList<Metadata>();
	}

	public void addOrReplaceMetadata(Metadata m) {
		if (mMetadata.contains(m)) {
			Metadata oldMeta = mMetadata.get(mMetadata.indexOf(m));
			if (oldMeta.getTimestamp().equals(m.getTimestamp())) {
				return;
			}
		}
		for (int i = 0; i < mMetadata.size(); i++) {
			Metadata meta = mMetadata.get(i);
			if (meta.getName().equalsIgnoreCase(m.getName())) {
				if (m.isSingleValue()) {
					mMetadata.remove(meta);
				}
			}
		}

		mMetadata.add(m);
	}

	public boolean remove(Metadata meta) {
		return mMetadata.remove(meta);
	}

	public ArrayList<Metadata> getMetadataAsCopy() {
		// TODO make a real copy here
		return this.getMetadata();
	}

	public ArrayList<Metadata> getMetadata() {
		return mMetadata;
	}

	public abstract boolean equals(Object o);

	public abstract int hashCode();
	// public abstract E clone();

}
