/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.datastructure
 * File:    Link.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.datastructure;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.UUID;
/**
 * Represents a Link
 * @author Tobias
 *
 */
public class Link {

	private String name;
	private Identifier i1;	
	private Identifier i2;
	//The metadata is stored in a hashmap
	private Hashtable<String, Hashtable<String, String>> meta;
	// Also the attributes
	private Hashtable<String, String> attr;
	private String hash = null;
	private boolean isSingleValue = true;
	
	public Link(String name, Identifier i1, Identifier i2){
		this.name = name;
		this.i1 = i1;
		this.i2 = i2;
		meta = new Hashtable<String, Hashtable<String, String>>();
		hash = UUID.randomUUID().toString();
	}
	
	public void addMeta(String name, String key, String value){
		Hashtable<String, String> tmp = new Hashtable<String, String>();
		tmp.put(key, value);
//		System.err.println("add meta: ("+name+") ("+key+") ("+value+")");
		this.meta.put(name, tmp);
	}
	
	public String getHash(){
		return hash;
	}

	public void setSingleValue(boolean b){
		isSingleValue = b;
	}

	public boolean isSingleValue(){
		return isSingleValue;
	}
	
	public String getUniqueLinkName(){
		StringBuffer reader = new StringBuffer();
		reader.append(name+"\r\n");
		reader.append(meta.toString());
		reader.append("\r\n["+i1.getName()+" "+i1.getAttributes().toString()+"] ");
		reader.append("["+i2.getName()+" "+i2.getAttributes().toString()+"]");
		return reader.toString();
	}	
	
	public void addAttribute(String key, String value){
		if(attr == null)
			attr = new Hashtable<String, String>();
		attr.put(key, value);
	}
	
	public String getAttribute(String key){
		return attr.get(key);
	}
	
	public boolean equals(Link l){
		String s1 = l.getName()+l.getIdentifier1().getUniqueIdentifierName()+l.getIdentifier2().getUniqueIdentifierName();
		String s2 = getName()+getIdentifier1().getUniqueIdentifierName()+getIdentifier2().getUniqueIdentifierName();
		return s1.equals(s2);		
	}
	
	public String toString(){
		StringBuffer buf = new StringBuffer();
		String s2;
		if(i2 == null) {
			s2 = "-";
		}
		else {
			s2 = i2.getName();
		}
		
		buf.append(name + "[");
		buf.append(i1.getName() + " / " + s2 + "] ");
		Enumeration<String> at = this.attr.keys();
		
		if(at.hasMoreElements()){
			while(at.hasMoreElements()){
				String key = at.nextElement();
				buf.append("["+key+"="+attr.get(key));
				buf.append("]\r\n");
		 }			
		}									
		Enumeration<String> iter = meta.keys();
		
		while(iter.hasMoreElements()){
			String key = iter.nextElement();
			buf.append("	"+key);
			buf.append(" [");
			Hashtable<String, String> values = meta.get(key);
			Enumeration<String> attributes = values.keys();
			
			while(attributes.hasMoreElements()){
				String key2 = attributes.nextElement();
				String value2 = values.get(key2);
				buf.append(key2 + "="+value2 + " ");
			}
			buf.append("]\r\n");
		}		
		
		return buf.toString();
	}
	
	public void setIdentifier1(Identifier i1) {
		this.i1 = i1;
	}

	public void setIdentifier2(Identifier i2) {
		this.i2 = i2;
	}
	
	public Hashtable<String, Hashtable<String, String>> getMetadata(){
		return meta;
	}
	
	public Hashtable<String, String> getAttributes(){
		return attr;
	}
	
	public void clearMeta(){
		this.meta.clear();
	}
	
	public Identifier getIdentifier1(){
		return i1;
	}
	
	public Identifier getIdentifier2(){
		return i2;
	}
	
	public String getName(){
		return name;
	}
	
}
