/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.datastructure
 * File:    MetadataRepository.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.datastructure;

import java.util.ArrayList;

public class MetadataRepository {

	private MultiMap<Identifier2, Link2> mMapIdentifier;
	private MultiMap<Link2, Identifier2> mMapLinks;

	public MetadataRepository() {
		mMapIdentifier = new MultiHashMap<Identifier2, Link2>();
		mMapLinks = new MultiHashMap<Link2, Identifier2>();
	}

	private void removeLink(Link2 link) {
		if (link != null) {
			Identifier2 i1 = link.getIdentifier1();
			Identifier2 i2 = link.getIdentifier2();
			if (i1 != null) {
				ArrayList<Link2> links = mMapIdentifier.get(i1);
				if (links == null) {
					mMapIdentifier.remove(i1);
				} else {
					links.remove(link);
					if (links.isEmpty()) {
						mMapIdentifier.remove(i1);
					}
				}
			}
			if (i2 != null) {
				ArrayList<Link2> links = mMapIdentifier.get(i2);
				if (links == null) {
					mMapIdentifier.remove(i2);
				} else {
					links.remove(link);
					if (links.isEmpty()) {
						mMapIdentifier.remove(i2);
					}
				}
			}
			mMapLinks.remove(link);
		}
	}

	private void addLink(Link2 link) {
		if (link != null) {
			// Identifier already exists?
			Identifier2 i1 = link.getIdentifier1();
			Identifier2 i2 = link.getIdentifier2();
			ArrayList<Link2> links1 = mMapIdentifier.get(i1);
			ArrayList<Link2> links2 = mMapIdentifier.get(i2);

			if (links1 == null || !links1.contains(link)) {
				mMapIdentifier.put(i1, link);
			} else {
				int index = links1.indexOf(link);
				Link2 l1 = links1.get(index);
				for (Metadata m : link.getMetadataAsCopy()) {
					l1.addOrReplaceMetadata(m);
				}
			}

			if (links2 == null || !links2.contains(link)) {
				mMapIdentifier.put(i2, link);
			} else {
				int index = links2.indexOf(link);
				Link2 l2 = links2.get(index);
				for (Metadata m : link.getMetadataAsCopy()) {
					l2.addOrReplaceMetadata(m);
				}
			}
			if (mMapLinks.hasKey(link)) {
				Link2 key = mMapLinks.getKey(link);
				for (Metadata m : link.getMetadataAsCopy()) {
					key.addOrReplaceMetadata(m);
				}
			}

			mMapLinks.put(link, i1);
			mMapLinks.put(link, i2);
		}		
	}

	public synchronized void addEntity(IfmapDataType entity){
		if(entity instanceof Identifier2){
			addIdentifier((Identifier2)entity);
		}
		else if (entity instanceof Link2){
			addLink((Link2)entity);
		}
	}
	
	public synchronized void removeEntity(IfmapDataType entity){
		if(entity instanceof Identifier2){
			removeIdentifier((Identifier2)entity);
		}
		else if (entity instanceof Link2){
			removeLink((Link2)entity);
		}
	}
	
	private void addIdentifier(Identifier2 ident) {
		if (ident != null) {
			if (!mMapIdentifier.hasKey(ident)) {
				mMapIdentifier.put(ident, null);
			} else {
				Identifier2 identRef = mMapIdentifier.getKey(ident);
				for (Metadata meta : ident.getMetadata()) {
					identRef.addOrReplaceMetadata(meta);
				}
			}			
		}		
	}
	
	private void removeIdentifier(Identifier2 ident){
		if(ident != null && this.mMapIdentifier.hasKey(ident)){
			Identifier2 ident_old = this.mMapIdentifier.getKey(ident);
			for(Metadata meta : ident.getMetadata()){
				ident_old.remove(meta);
			}
			if(ident_old.getMetadata().isEmpty()){
				mMapIdentifier.remove(ident_old);
			}			
		}
	}	
	public void printMaps(){
		System.err.println("MultiMap<Identifier, Link>\r\n"+this.mMapIdentifier.toString());
		System.err.println("MultiMap<Link, Identifier>\r\n"+this.mMapLinks.toString());
	}
}