/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.visual
 * File:    SubscribeObservable.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.visual;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Observable;

import javax.swing.AbstractButton;
import javax.swing.JFrame;

import org.trustedcomputinggroup._2010.ifmap._2.AccessRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.DeviceType;
import org.trustedcomputinggroup._2010.ifmap._2.IPAddressType;
import org.trustedcomputinggroup._2010.ifmap._2.IdentityType;
import org.trustedcomputinggroup._2010.ifmap._2.MACAddressType;

public class SubscribeObservable extends Observable {
	
	private Subscription_GUI gui;
	private Object result;
	
	public SubscribeObservable(){
		gui = new Subscription_GUI();
		gui.setCallback(this);
	}
	
	public void showGUI(){
		gui.setVisible(true);			
	}
	
	private void setIdentifier(Object o){
		this.result = o;
		setChanged();
	    notifyObservers();
	}
	
	public void update(){
		if(gui.rdbtnIpaddress.isSelected()){			
			if(gui.ipAddressPanel.ipAddressRadioButton.isSelected()){
				String adr = gui.ipAddressPanel.ipAddressField.getText().trim();	
				if(!adr.equals("")){					
					IPAddressType ip = new IPAddressType();
					ip.setType("IPv4");						
					ip.setValue(adr);					
					setIdentifier(ip);
				}
			}
		}
		else if(gui.rdbtnMacaddress.isSelected()){
			String mac_val = gui.macaddresspanel.value.getText().trim();
			String mac_admin = gui.macaddresspanel.admin.getText().trim();
			if(!mac_val.equals("")){
				MACAddressType mac = new MACAddressType();
				mac.setValue(mac_val);
				if(!mac_admin.equals("")){
					mac.setAdministrativeDomain(mac_admin);
				}
				setIdentifier(mac);
			}
		}
		else if(gui.rdbtnAccessrequest.isSelected()){
			String ar_name = gui.accessrequestpanel.name.getText().trim();
			String ar_admin = gui.accessrequestpanel.admin.getText().trim();
			if(!ar_name.equals("")){
				AccessRequestType ar = new AccessRequestType();
				ar.setName(ar_name);
				if(!ar_admin.equals("")){
					ar.setAdministrativeDomain(ar_admin);
				}
				setIdentifier(ar);
			}
		}
		else if(gui.rdbtnDevice.isSelected()){
			String d_name = gui.devicepanel.name.getText().trim();
			String d_aik = gui.devicepanel.aik.getText().trim();
			if(!d_name.equals("")){
				DeviceType dt = new DeviceType();
				dt.setName(d_name);
				if(!d_aik.equals("")){
					dt.setAikName(d_aik);
				}
				setIdentifier(dt);
			}
		}
		else if(gui.rdbtnIdentity.isSelected()){
			String id_name = gui.identitypanel.name.getText().trim();			
			if(!id_name.equals("")){
				String type = gui.identitypanel.jComboBox1.getSelectedItem().toString();
				String admin = gui.identitypanel.admin.getText().trim();
				IdentityType id = new IdentityType();
				id.setType(type);
				id.setName(id_name);				
				if(!admin.equals("")){
					id.setAdministrativeDomain(admin);
				}
				setIdentifier(id);
			}
		}
	}
	
	public Object getIdentifier(){
		return result;
	}
	
	public long getDepth(){
		long l = 0;
		try{
			l = Long.parseLong(String.valueOf(gui.jSpinner1.getValue()));
		}
		catch(NumberFormatException err){
			return l;
		}
		return l;
	}
	
	public long getSize(){
		long l = 0;
		try{
			l = Long.parseLong(String.valueOf(gui.jSpinner2.getValue()));
		}
		catch(NumberFormatException err){
			return l;
		}
		return l;
	}
	
	public String getFilter(){
		return gui.jTextField1.getText();
	}
	
	public String getLinks(){
		return gui.jTextField2.getText();
	}
	
	public String[] getTerminals(){
		ArrayList<String> arr = new ArrayList<String>();
		
		if(gui.radioButton.isSelected())
			arr.add(gui.radioButton.getText());
		if(gui.radioButton_1.isSelected())
			arr.add(gui.radioButton_1.getText());
		if(gui.radioButton_2.isSelected())
			arr.add(gui.radioButton_2.getText());
		if(gui.radioButton_3.isSelected())
			arr.add(gui.radioButton_3.getText());
		if(gui.radioButton_4.isSelected())
			arr.add(gui.radioButton_4.getText());				
		
		String[] arrString = new String[arr.size()];
		
		for(int i = 0; i < arr.size(); i++){
			arrString[i] = arr.get(i);
		}
		arr = null;
		return arrString;		
	}
	
	public void center(){
		gui.setLocationRelativeTo(null);
	}
	
}
